'use strict';

import GObject from 'gi://GObject';
import St from 'gi://St';
import Gio from 'gi://Gio';
import GLib from 'gi://GLib';
import Soup from 'gi://Soup';
import Clutter from 'gi://Clutter';
import * as Main from 'resource:///org/gnome/shell/ui/main.js';
import * as PanelMenu from 'resource:///org/gnome/shell/ui/panelMenu.js';
import * as PopupMenu from 'resource:///org/gnome/shell/ui/popupMenu.js';
import {Extension} from 'resource:///org/gnome/shell/extensions/extension.js';

const GETTEXT_DOMAIN = 'currency-tracker';
const INDICATOR_ICON = 'currency-tracker-symbolic';

const CURRENCY_PAIRS = {
    'USD-EUR': 'USD/EUR',
    'USD-GBP': 'USD/GBP',
    'USD-CNY': 'USD/CNY',
    'USD-TRY': 'USD/TRY',
    'EUR-USD': 'EUR/USD',
    'EUR-GBP': 'EUR/GBP',
    'EUR-CNY': 'EUR/CNY',
    'EUR-TRY': 'EUR/TRY',
    'GBP-TRY': 'GBP/TRY',
    'GBP-CNY': 'GBP/CNY',
    'CNY-EUR': 'CNY/EUR',
    'CNY-USD': 'CNY/USD',
    'BTC-EUR': 'BTC/EUR',
    'BTC-USD': 'BTC/USD'
};

const CURRENCY_CATEGORIES = {
    'USD Pairs': ['USD-EUR', 'USD-GBP', 'USD-CNY', 'USD-TRY'],
    'EUR Pairs': ['EUR-USD', 'EUR-GBP', 'EUR-CNY', 'EUR-TRY'],
    'CNY Pairs': ['CNY-USD', 'CNY-EUR'],
    'TRY Pairs': ['USD-TRY', 'EUR-TRY'],
    'Bitcoin': ['BTC-USD', 'BTC-EUR']
};

const Indicator = GObject.registerClass(
class Indicator extends PanelMenu.Button {
    _init() {
        super._init(0.0, 'Currency Tracker');
        
        this._panelBox = new St.BoxLayout({
            style_class: 'panel-status-menu-box'
        });
        
        this._icon = new St.Icon({
            gicon: this._getCustomIcon(),
            style_class: 'system-status-icon currency-tracker-icon'
        });
        
        this._label = new St.Label({
            text: 'Loading...',
            y_align: Clutter.ActorAlign.CENTER,
            style_class: 'currency-tracker-label'
        });
        
        this._panelBox.add_child(this._icon);
        this._panelBox.add_child(this._label);
        this.add_child(this._panelBox);
        
        this._buildMenu();
        this._refresh();
    }

    _buildMenu() {
        // Kategorilere göre para birimi seçimi
        for (const [category, pairs] of Object.entries(CURRENCY_CATEGORIES)) {
            const categorySubMenu = new PopupMenu.PopupSubMenuMenuItem(category);
            
            for (const pair of pairs) {
                const menuItem = new PopupMenu.PopupMenuItem(CURRENCY_PAIRS[pair]);
                menuItem.connect('activate', () => {
                    this._currentPair = pair;
                    this._refresh();
                });
                categorySubMenu.menu.addMenuItem(menuItem);
            }
            
            this.menu.addMenuItem(categorySubMenu);
        }
        
        this.menu.addMenuItem(new PopupMenu.PopupSeparatorMenuItem());
        
        const refreshItem = new PopupMenu.PopupMenuItem('Refresh');
        refreshItem.connect('activate', () => {
            this._refresh();
        });
        this.menu.addMenuItem(refreshItem);
    }

    async _refresh() {
        try {
            this._label.set_text('Loading...');
            const pair = this._currentPair || 'USD-TRY';
            const response = await this._fetchData(pair);
            
            if (!response) {
                throw new Error('No response from API');
            }

            const text = new TextDecoder().decode(response);
            const data = JSON.parse(text);
            const currencyData = data[pair.replace('-', '')];
            
            if (!currencyData) {
                throw new Error('Invalid currency data');
            }

            let displayText = `${CURRENCY_PAIRS[pair]}: ${currencyData.bid}`;
            if (currencyData.pctChange) {
                const change = parseFloat(currencyData.pctChange);
                const changeText = change >= 0 ? `+${change}%` : `${change}%`;
                displayText += ` (${changeText})`;
            }
            this._label.set_text(displayText);
        } catch (error) {
            console.error('Refresh error:', error);
            this._label.set_text('Error: ' + error.message);
        }
    }

    async _fetchData(pair) {
        let session = null;
        try {
            const url = `https://economia.awesomeapi.com.br/json/last/${pair}`;
            session = new Soup.Session();
            const message = Soup.Message.new('GET', url);
            
            if (!message) {
                throw new Error('Failed to create HTTP message');
            }

            const bytes = await session.send_and_read_async(
                message,
                GLib.PRIORITY_DEFAULT,
                null
            );

            if (!bytes) {
                throw new Error('No response data');
            }

            return bytes.get_data();
        } catch (error) {
            console.error('Fetch error:', error);
            return null;
        } finally {
            if (session) {
                session.abort();
            }
        }
    }
});

export default class CurrencyTracker extends Extension {
    enable() {
        console.log('Currency Tracker: enabling...');
        this._indicator = new Indicator();
        Main.panel.addToStatusArea('currency-tracker', this._indicator);
    }

    disable() {
        console.log('Currency Tracker: disabling...');
        if (this._indicator) {
            this._indicator.destroy();
            this._indicator = null;
        }
    }
}