'use strict';

import GObject from 'gi://GObject?version=2.0';
import Gio from 'gi://Gio?version=2.0';
import GLib from 'gi://GLib?version=2.0';
import Soup from 'gi://Soup?version=3.0';
import St from 'gi://St?version=1.0';
import Clutter from 'gi://Clutter?version=1.0';

import * as Main from 'resource:///org/gnome/shell/ui/main.js';
import * as PanelMenu from 'resource:///org/gnome/shell/ui/panelMenu.js';
import * as PopupMenu from 'resource:///org/gnome/shell/ui/popupMenu.js';

import {Extension, gettext as _} from 'resource:///org/gnome/shell/extensions/extension.js';
import * as ByteArray from 'resource:///org/gnome/gjs/bytearray.js';

const GETTEXT_DOMAIN = 'currency-tracker';

const ExtensionUtils = Extension;

const CURRENCY_PAIRS = {
    'USD-EUR': 'USD/EUR',
    'USD-GBP': 'USD/GBP',
    'USD-CNY': 'USD/CNY',
    'USD-TRY': 'USD/TRY',
    'EUR-USD': 'EUR/USD',
    'EUR-GBP': 'EUR/GBP',
    'EUR-CNY': 'EUR/CNY',
    'EUR-TRY': 'EUR/TRY',
    'GBP-TRY': 'GBP/TRY',
    'GBP-CNY': 'GBP/CNY',
    'CNY-TRY': 'CNY/TRY',
    'CNY-EUR': 'CNY/EUR',
    'CNY-USD': 'CNY/USD',
    'CNY-GBP': 'CNY/GBP',
    'BTC-TRY': 'BTC/TRY',
    'BTC-EUR': 'BTC/EUR',
    'BTC-GBP': 'BTC/GBP',
    'BTC-USD': 'BTC/USD',
    'BTC-CNY': 'BTC/CNY'
};

const CURRENCY_CATEGORIES = {
    'USD Pairs': ['USD-EUR', 'USD-GBP', 'USD-CNY', 'USD-TRY'],
    'EUR Pairs': ['EUR-USD', 'EUR-GBP', 'EUR-CNY', 'EUR-TRY'],
    'CNY Pairs': ['CNY-USD', 'CNY-EUR', 'CNY-GBP', 'CNY-TRY'],
    'TRY Pairs': ['USD-TRY', 'EUR-TRY', 'GBP-TRY', 'CNY-TRY'],
    'Bitcoin': ['BTC-USD', 'BTC-EUR', 'BTC-GBP', 'BTC-TRY', 'BTC-CNY']
};

const INDICATOR_ICON = 'currency-tracker-symbolic';

const Indicator = GObject.registerClass(
class Indicator extends PanelMenu.Button {
    _init(settings) {
        super._init(0.0, 'Currency Tracker');

        this._settings = settings;
        
        // Settings değişikliklerini dinle
        this._settingsChangedId = this._settings.connect('changed', () => {
            this._refresh();
        });
        
        this._panelBox = new St.BoxLayout({
            style_class: 'panel-status-menu-box'
        });
        
        this._icon = new St.Icon({
            icon_name: INDICATOR_ICON,
            style_class: 'system-status-icon currency-tracker-icon'
        });
        
        if (!this._settings.get_boolean('show-icon')) {
            this._icon.hide();
        }
        
        this._buttonText = new St.Label({
            text: 'Loading...',
            y_align: Clutter.ActorAlign.CENTER,
            style_class: 'currency-tracker-label'
        });
        
        this._panelBox.add_child(this._icon);
        this._panelBox.add_child(this._buttonText);
        this.add_child(this._panelBox);
        
        this._buildMenu();
        
        this._updateTimeout = null;
        this._refresh();
    }

    _buildMenu() {
        for (const [category, pairs] of Object.entries(CURRENCY_CATEGORIES)) {
            const categorySubMenu = new PopupMenu.PopupSubMenuMenuItem(_(category));
            
            for (const pair of pairs) {
                const menuItem = new PopupMenu.PopupMenuItem(CURRENCY_PAIRS[pair]);
                menuItem.connect('activate', () => {
                    this._settings.set_string('currency-pair', pair);
                    this._refresh();
                });
                categorySubMenu.menu.addMenuItem(menuItem);
            }
            
            this.menu.addMenuItem(categorySubMenu);
        }
        
        this.menu.addMenuItem(new PopupMenu.PopupSeparatorMenuItem());
        
        const refreshItem = new PopupMenu.PopupMenuItem(_('Refresh Now'));
        refreshItem.connect('activate', () => this._refresh());
        this.menu.addMenuItem(refreshItem);
    }

    async _refresh() {
        if (this._updateTimeout) {
            GLib.source_remove(this._updateTimeout);
            this._updateTimeout = null;
        }

        try {
            const pair = this._settings.get_string('currency-pair');
            const response = await this._fetchData(pair);
            
            if (response) {
                const data = JSON.parse(response);
                const currencyData = data[pair.replace('-', '')];
                
                if (currencyData) {
                    let displayText = `${CURRENCY_PAIRS[pair]}: ${currencyData.bid}`;
                    if (currencyData.pctChange) {
                        const change = parseFloat(currencyData.pctChange);
                        const changeText = change >= 0 ? `+${change}%` : `${change}%`;
                        displayText += ` (${changeText})`;
                    }
                    this._buttonText.set_text(displayText);
                }
            }
        } catch (error) {
            logError(error);
            this._buttonText.set_text('Error');
        }

        this._updateTimeout = GLib.timeout_add_seconds(
            GLib.PRIORITY_DEFAULT,
            this._settings.get_int('refresh-interval'),
            () => {
                this._refresh();
                return GLib.SOURCE_CONTINUE;
            }
        );
    }

    async _fetchData(pair) {
        try {
            const url = `https://economia.awesomeapi.com.br/json/last/${pair}`;
            const session = new Soup.Session();
            const message = Soup.Message.new('GET', url);
            
            if (!message) {
                throw new Error('Failed to create HTTP message');
            }

            const bytes = await session.send_and_read_async(
                message,
                GLib.PRIORITY_DEFAULT,
                null
            );

            if (!bytes) {
                throw new Error('No response data');
            }

            const data = bytes.get_data();
            if (!data) {
                throw new Error('Empty response data');
            }

            return ByteArray.toString(data);
        } catch (error) {
            logError(error);
            return null;
        } finally {
            if (session) {
                session.abort();
            }
        }
    }

    destroy() {
        if (this._updateTimeout) {
            GLib.source_remove(this._updateTimeout);
            this._updateTimeout = null;
        }
        
        if (this._settingsChangedId) {
            this._settings.disconnect(this._settingsChangedId);
            this._settingsChangedId = null;
        }
        
        super.destroy();
    }
});

export default class CurrencyTracker extends Extension {
    enable() {
        this._settings = this.getSettings();
        
        if (!this._indicator) {
            this._indicator = new Indicator(this._settings);
            Main.panel.addToStatusArea('currency-tracker', this._indicator);
        }
    }

    disable() {
        if (this._indicator) {
            this._indicator.destroy();
            this._indicator = null;
        }
        
        if (this._settings) {
            this._settings = null;
        }
    }
}